/***********************************************************
 * $Id: LocalizedComparatorFactory.java 20 2007-03-01 10:45:50Z wglas $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: Feb 28, 2007
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/

package org.clazzes.util.comparators;

import java.text.Collator;
import java.util.Comparator;
import java.util.Locale;

/**
 * This ComparatorFactory implementation returns a localized {@link Collator} 
 * instance for String objects and an ComparableComparator instance for
 * Comparables.
 * 
 * @author wglas
 */
public class LocalizedComparatorFactory extends DefaultComparatorFactory {

    private Collator collator;
    
    /**
     * Construct a LocalizedComparatorFactory for the current locale and
     * the default strength {@link Collator#IDENTICAL}.
     * 
     * @see Collator#getInstance()
     */
    public LocalizedComparatorFactory()
    {
        this.collator = Collator.getInstance();
        this.collator.setStrength(Collator.IDENTICAL);
    }
    
    /**
     * Construct a LocalizedComparatorFactory for the given locale and
     * the default strength {@link Collator#IDENTICAL}.
     * 
     * @param locale The local for which to contruct the string Collator.
     * 
     * @see Collator#getInstance(Locale)
     */
    public LocalizedComparatorFactory(Locale locale)
    {
        this.collator = Collator.getInstance(locale);
        this.collator.setStrength(Collator.IDENTICAL);
    }
    
    /**
     * Construct a LocalizedComparatorFactory for the given locale and
     * the default strength {@link Collator#IDENTICAL}.
     * 
     * @param locale The local for which to contruct the string Collator.
     * @param strength The strength of the underlying Collator. One of
     * <ul><li>{@link Collator#PRIMARY}</li>
     *     <li>{@link Collator#SECONDARY}</li>
     *     <li>{@link Collator#TERTIARY}</li>
     *     <li>{@link Collator#IDENTICAL}</li>
     * </ul>
     * 
     * @see Collator#getInstance(Locale)
     * @see Collator#setStrength(int)
     */
    public LocalizedComparatorFactory(Locale locale, int strength)
    {
        this.collator = Collator.getInstance(locale);
        this.collator.setStrength(strength);
    }

    /* (non-Javadoc)
     * @see org.clazzes.util.comparators.DefaultComparatorFactory#getComparator(java.lang.Class, boolean)
     */
    @Override
    public Comparator getComparator(Class clazz) {
       
        if (String.class.isAssignableFrom(clazz))
            return this.collator;
       
        return super.getComparator(clazz);
    }

    /**
     * @return The collator used for string comparisons.
     */
    public Collator getCollator() {
        return this.collator;
    }

    /**
     * @param collator The collator for string comparisons to set.
     */
    public void setCollator(Collator collator) {
        this.collator = collator;
    }
    
}
