/***********************************************************
 * $Id: UtcTimestamp.java 23 2007-03-14 07:51:31Z rbreuss $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 2006-03-13
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/

package org.clazzes.util.datetime;

import java.io.Serializable;
import java.util.Calendar;
import java.util.TimeZone;

import javax.persistence.Embeddable;

/**
 * A class to store UTC timestamps and timezone offset to database.
 * 
 * @author rbreuss
 * 
 */
@Embeddable
public class UtcTimestamp implements Serializable {

    /**
     * To be changed at class-modifcation.
     */
    private static final long serialVersionUID = -4171006885461716579L;

    /**
     * This class is designed to handle UTC-Timestamps, so this field must not
     * be changed. It is used internally to calculate appropreate Calendar
     * Objects.
     */
    private static TimeZone utcTimezone = TimeZone.getTimeZone("UTC");

    /**
     * The amount of milliseconds elapsed since 1970-01-01.
     */
    private Long utcMillis;

    /**
     * The timezone offset in minutes.
     */
    private Integer tzOffset;

    /**
     * Construct a new UtcTimestamp for 1970-01-01 (tzOffset=0).
     */
    public UtcTimestamp() {
        this(0L, 0);
    }
    /** Construct a new UtcTimestamp for the passed Calendar object.
     * @param c
     */
    public UtcTimestamp(Calendar c) {
        super();
        this.setFromCalendar(c);
    }
    /**
     * Construct a new UtcTimestamp for the passed milliseconds with a tzOffset=0.
     * @param millis
     */
    public UtcTimestamp(long millis) {
        this(millis, 0);
    }
    /**
     * Construct a new UtcTimestamp for the passed milliseconds an timezone offset.
     * @param millis
     * @param tzOffset in minutes
     */
    public UtcTimestamp(long millis, int tzOffset) {
        this.utcMillis = Long.valueOf(millis);
        this.tzOffset = Integer.valueOf(tzOffset);
    }
    

    /**
     * Convert this UtcTimestamp to java.util.Calendar.
     * @return
     */
    public Calendar toCalendar() {
        if (this.utcMillis == null || this.tzOffset == null)
        {
            return null;
        }
        else
        {
            Calendar c = Calendar.getInstance(UtcTimestamp.utcTimezone);
            c.setTimeInMillis(this.utcMillis.longValue() + (long) this.tzOffset.intValue() * 60000);
            c.set(Calendar.ZONE_OFFSET, this.tzOffset.intValue() * 60000);
            return c;
        }
    }
    
    /** 
     * Convert the passed Calendar object to UtcTimestamp.
     * @param c
     */
    public void setFromCalendar(Calendar c) {
        if (c != null) 
        {
            this.utcMillis = Long.valueOf(c.getTimeInMillis());
            this.tzOffset = Integer.valueOf((c.get(Calendar.ZONE_OFFSET) + c.get(Calendar.DST_OFFSET)) / 60000);
        }
        else
        {
            this.utcMillis = null;
            this.tzOffset = null;
        }
    }
    
    /**
     * @return Returns the tzOffset.
     */
    public Integer getTzOffset() {
        return this.tzOffset;
    }

    /**
     * @param tzOffset
     *            The tzOffset to set.
     */
    public void setTzOffset(Integer tzOffset) {
        this.tzOffset = tzOffset;
    }

    /**
     * @return Returns the utcMillis.
     */
    public Long getUtcMillis() {
        return this.utcMillis;
    }

    /**
     * @param utcMillis
     *            The utcMillis to set.
     */
    public void setUtcMillis(Long utcMillis) {
        this.utcMillis = utcMillis;
    }

}
